@@ -1,9 +1,12 @@
 from __future__ import with_statement
 
+import os
+import re
+
 from fabric.api import run, sudo
 from fabric.contrib.project import rsync_project
+
 from chef_solo_cup.log import setup_custom_logger
-import os
 
 
 def get_hosts(args, logger=None):
@@ -11,6 +14,14 @@ def get_hosts(args, logger=None):
 
     hosts = {}
 
+    includes = []
+    if args.dna_patterns:
+        includes = map(lambda x: re.compile(x), args.dna_patterns)
+
+    excludes = []
+    if args.exclude:
+        excludes = map(lambda x: re.compile(x), args.exclude)
+
     for root, sub_folders, files in os.walk(dna_path):
         files = filter(lambda f: ".json" in f, files)
         for f in files:
@@ -19,15 +30,18 @@ def get_hosts(args, logger=None):
             provider = path.pop()
             service = path.pop()
 
-            if args.dna_patterns:
-                skip = True
-                for dna in args.dna_patterns:
-                    if dna in f:
-                        skip = False
-
+            if len(excludes):
+                skip = map(lambda regex: regex.search(f), excludes)
+                skip = reduce(lambda x, y: x or y, skip)
                 if skip:
                     continue
 
+            if len(includes):
+                skip = map(lambda regex: regex.search(f), includes)
+                skip = reduce(lambda x, y: x or y, skip)
+                if skip is None:
+                    continue
+
             if args.regions and region not in args.regions:
                 continue
             if args.providers and provider not in args.providers:
@@ -58,7 +72,7 @@ def rsync_project_dry(args, logger=None, **kwargs):
         logger = setup_custom_logger('chef-solo-cup', args)
 
     if args.dry_run:
-        logger.info("[RSYNC_PROJECT] {0}".format(**kwargs))
+        logger.info("[RSYNC_PROJECT] From {0} to {1} with opts='{2}' excluding='{3}'".format(kwargs.get('local_dir'), kwargs.get('remote_dir'), kwargs.get('extra_opts'), kwargs.get('exclude')))
     else:
         rsync_project(**kwargs)
 
